<?php

namespace Drp\Analytics\Foundation\Handlers;

use Drp\Analytics\Contracts\HandlesAnalytics;
use Drp\Analytics\Foundation\AnalyticPayload;
use Drp\Analytics\Foundation\AnalyticReference;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Arr;

class EloquentMetric implements HandlesAnalytics
{
    // phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
    public function store(AnalyticReference $reference, AnalyticPayload $payload): void
    {
        $model = config('analytics.models.metric');
        $metric = tap($model::firstOrCreate(['event' => $reference->getEvent()]))->increment('count');

        if (config('analytics.metric-store-analyticables') === false) {
            return;
        }

        if (in_array($reference->getAnalyticable(), (array) $metric->payload, false)) {
            return;
        }

        $metric->update(['payload' => Arr::prepend((array) $metric->payload, $reference->getAnalyticable())]);
    }

    public function get(string $event, ?callable $query = null): Collection
    {
        $model = config('analytics.models.metric');

        $baseQuery = $model::query()->whereEvent($event);

        if ($query === null) {
            return $baseQuery->get();
        }

        return $query($baseQuery)->get();
    }
}
